﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Infragistics.Windows.OutlookBar;
using System.ComponentModel;
using System.Windows.Data;
using System.Collections;

namespace Infragistics.Extensions
{
	public class OutlookBarExtensions
	{
		#region IsSynchronizedWithCurrentItem

		/// <summary>
		/// IsSynchronizedWithCurrentItem Attached Dependency Property
		/// </summary>
		public static readonly DependencyProperty IsSynchronizedWithCurrentItemProperty =
			DependencyProperty.RegisterAttached("IsSynchronizedWithCurrentItem", typeof(bool?), typeof(OutlookBarExtensions),
				new FrameworkPropertyMetadata((bool?)null,
					new PropertyChangedCallback(OnIsSynchronizedWithCurrentItemChanged)));

		/// <summary>
		/// Gets a boolean indicating whether the SelectedGroup is synchronized with the current item
		/// </summary>
		[AttachedPropertyBrowsableForType(typeof(XamOutlookBar))]
		public static bool? GetIsSynchronizedWithCurrentItem(DependencyObject d)
		{
			return (bool?)d.GetValue(IsSynchronizedWithCurrentItemProperty);
		}

		/// <summary>
		/// Sets a boolean indicating whether the SelectedGroup is synchronized with the current item
		/// indicates ....
		/// </summary>
		public static void SetIsSynchronizedWithCurrentItem(DependencyObject d, bool? value)
		{
			d.SetValue(IsSynchronizedWithCurrentItemProperty, value);
		}

		/// <summary>
		/// Handles changes to the IsSynchronizedWithCurrentItem property.
		/// </summary>
		private static void OnIsSynchronizedWithCurrentItemChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var outlookBar = d as XamOutlookBar;

			if (outlookBar == null)
				return;

			var syncHelper = GetCurrentSynchronizationHelper(d);
			if (syncHelper == null)
				SetCurrentSynchronizationHelper(d, syncHelper = new SynchronizationHelper(outlookBar));

			syncHelper.InitializeIsSynchronizedResolved();
		}
		#endregion //IsSynchronizedWithCurrentItem

		#region CurrentSynchronizationHelper

        private static readonly DependencyProperty CurrentSynchronizationHelperProperty =
            DependencyProperty.RegisterAttached("CurrentSynchronizationHelper", typeof(SynchronizationHelper), typeof(OutlookBarExtensions),
                new FrameworkPropertyMetadata((SynchronizationHelper)null));

        private static SynchronizationHelper GetCurrentSynchronizationHelper(DependencyObject d)
        {
            return (SynchronizationHelper)d.GetValue(CurrentSynchronizationHelperProperty);
        }

        private static void SetCurrentSynchronizationHelper(DependencyObject d, SynchronizationHelper value)
        {
            d.SetValue(CurrentSynchronizationHelperProperty, value);
        }

        #endregion // CurrentSynchronizationHelper

		#region SynchronizationHelper
		private class SynchronizationHelper : DependencyObject
			, IWeakEventListener
		{
			#region Member Variables

			private XamOutlookBar _outlookBar;
			private bool _isSynchronizedResolved;
			private ICollectionView _collectionView;
			private bool _isChangingCurrent;
			private bool _isSynchronizingCurrent;

			#endregion //Member Variables

			#region Constructor
			internal SynchronizationHelper(XamOutlookBar outlookBar)
			{
				_outlookBar = outlookBar;

				if (!_outlookBar.IsInitialized)
					_outlookBar.Initialized += new EventHandler(OnOutlookBarInitialized);

				// watch for when the selectedgroup changes
				outlookBar.SelectedGroupChanged += new RoutedEventHandler(OnOutlookBarSelectedGroupChanged);

				// watch for when the current groups changes
				BindingOperations.SetBinding(this, CurrentGroupsSourceProperty, new Binding { Path = new PropertyPath(XamOutlookBar.GroupsSourceProperty), Mode = BindingMode.OneWay, Source = outlookBar });

				// in case the groups source was already set initialize the collection view
				this.InitializeCollectionView();

				if (_outlookBar.IsInitialized)
					this.InitializeIsSynchronizedResolved();
			}
			#endregion //Constructor

			#region Properties

			#region CollectionView
			private ICollectionView CollectionView
			{
				get { return _collectionView; }
				set
				{
					if (value != _collectionView)
					{
						if (null != _collectionView)
						{
							CurrentChangingEventManager.RemoveListener(_collectionView, this);
							CurrentChangedEventManager.RemoveListener(_collectionView, this);
						}

						_collectionView = value;

						if (null != _collectionView)
						{
							CurrentChangingEventManager.AddListener(_collectionView, this);
							CurrentChangedEventManager.AddListener(_collectionView, this);
						}

						this.InitializeIsSynchronizedResolved();
					}
				}
			}
			#endregion //CollectionView

			#region CurrentGroupsSource

			/// <summary>
			/// Identifies the <see cref="CurrentGroupsSource"/> dependency property
			/// </summary>
			private static readonly DependencyProperty CurrentGroupsSourceProperty = DependencyProperty.Register("CurrentGroupsSource",
				typeof(IEnumerable), typeof(SynchronizationHelper), new FrameworkPropertyMetadata(null, new PropertyChangedCallback(OnCurrentGroupsSourceChanged)));

			private static void OnCurrentGroupsSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
			{
				((SynchronizationHelper)d).InitializeCollectionView();
			}

			#endregion //CurrentGroupsSource

			#endregion //Properties

			#region Methods

			#region InitializeCollectionView
			private void InitializeCollectionView()
			{
				var collectionView = _collectionView;

				if (_outlookBar.GroupsSource != null)
					collectionView = CollectionViewSource.GetDefaultView(_outlookBar.GroupsSource);
				else
					collectionView = null;

				this.CollectionView = collectionView;
			}
			#endregion //InitializeCollectionView

			#region InitializeIsSynchronizedResolved
			internal void InitializeIsSynchronizedResolved()
			{
				bool? isSynchronized = GetIsSynchronizedWithCurrentItem(_outlookBar);
				bool wasSynchronized = _isSynchronizedResolved;
				bool shouldSynchronize = wasSynchronized;

				if (isSynchronized != null)
				{
					shouldSynchronize = _collectionView != null && isSynchronized.Value;
				}
				else
				{
					if (!_outlookBar.IsInitialized)
						return;

					shouldSynchronize = _collectionView != null && !CollectionViewSource.IsDefaultView(_collectionView);
				}

				_isSynchronizedResolved = shouldSynchronize;

				if (!wasSynchronized && shouldSynchronize)
				{
					if (_outlookBar.SelectedGroup != null)
						this.SetCurrentToSelected();
					else
						this.SetSelectedToCurrent();
				}
			}
			#endregion //InitializeIsSynchronizedResolved

			#region OnCurrentChanged
			private void OnCurrentChanged()
			{
				if (_isChangingCurrent)
					return;

				if (!_isSynchronizedResolved)
					return;

				_isChangingCurrent = true;

				try
				{
					this.SetSelectedToCurrent();
				}
				finally
				{
					_isChangingCurrent = false;
				}
			}
			#endregion //OnCurrentChanged

			#region OnCurrentChanging
			private void OnCurrentChanging(CurrentChangingEventArgs changingArgs)
			{
				if (_isChangingCurrent && changingArgs.IsCancelable)
					changingArgs.Cancel = true;
			}
			#endregion //OnCurrentChanging

			#region OnOutlookBarInitialized
			private void OnOutlookBarInitialized(object sender, EventArgs e)
			{
				_outlookBar.Initialized -= new EventHandler(OnOutlookBarInitialized);
				this.InitializeIsSynchronizedResolved();
			}
			#endregion //OnOutlookBarInitialized

			#region OnOutlookBarSelectedGroupChanged
			private void OnOutlookBarSelectedGroupChanged(object sender, RoutedEventArgs e)
			{
				if (_isSynchronizedResolved)
					this.SetCurrentToSelected();
			}
			#endregion //OnOutlookBarSelectedGroupChanged

			#region SetCurrentToSelected
			private void SetCurrentToSelected()
			{
				if (_isSynchronizingCurrent || !_isSynchronizedResolved)
					return;

				_isSynchronizingCurrent = true;

				try
				{
					var selected = _outlookBar.SelectedGroup;

					if (selected == null)
						_collectionView.MoveCurrentToPosition(-1);
					else
						_collectionView.MoveCurrentTo(selected.DataContext);
				}
				finally
				{
					_isSynchronizingCurrent = false;
				}
			}
			#endregion //SetCurrentToSelected

			#region SetSelectedToCurrent
			private void SetSelectedToCurrent()
			{
				if (_isSynchronizingCurrent || !_isSynchronizedResolved)
					return;

				_isSynchronizingCurrent = true;

				try
				{
					var item = _collectionView.CurrentItem;

					if (item == null)
						_outlookBar.SelectedGroup = null;
					else
					{
						OutlookBarGroup groupToSelect = null;

						foreach (OutlookBarGroup group in _outlookBar.Groups)
						{
							if (group.DataContext == item)
							{
								groupToSelect = group;
								break;
							}
						}

						_outlookBar.SelectedGroup = groupToSelect;
					}
				}
				finally
				{
					_isSynchronizingCurrent = false;
				}
			}
			#endregion //SetSelectedToCurrent

			#endregion //Methods

			#region IWeakEventListener Members
			bool IWeakEventListener.ReceiveWeakEvent(Type managerType, object sender, EventArgs e)
			{
				if (managerType == typeof(CurrentChangedEventManager))
				{
					this.OnCurrentChanged();
				}
				else if (managerType == typeof(CurrentChangingEventManager))
				{
					var changingArgs = e as CurrentChangingEventArgs;
					this.OnCurrentChanging(changingArgs);
				}
				else
				{
					return false;
				}

				return true;
			}
			#endregion //IWeakEventListener Members
		} 
		#endregion //SynchronizationHelper
	}
}
